;; Generated package description from literate-scratch.el  -*- mode: lisp-data; no-byte-compile: t -*-
(define-package "literate-scratch" "2.0.0.20250418.62743" "Lisp Interaction w/ text paragraphs" '((emacs "29.1")) :commit "2317f456e6047b4d595b7c5082dc8106dc1fc01d" :authors '(("Sean Whitton" . "spwhitton@spwhitton.name")) :maintainer '("Sean Whitton" . "spwhitton@spwhitton.name") :keywords '("lisp" "text") :url "https://git.spwhitton.name/dotfiles/tree/.emacs.d/site-lisp/literate-scratch.el")
